"""Owega's licence: DGPL-1.0."""
OwegaLicense = """\
#                     DARKGEEM PUBLIC LICENSE
#                      Version 1.0, May 2024
#
#By Darkgeem  <darkgeem AT pyrokinesis DOT fr>
#
#
#Everyone is permitted to copy and distribute verbatim or modified
#copies of this license document, and changing it is allowed as long
#as the name is changed.
#
#
#
#I. Definitions:
#
#THE FOLLOWING DEFINITION APPLY TO ALL INSTANCES OF SAID TERM IN THIS DOCUMENT:
#
#- "Articles" means, collectively, all articles written by author
#  which describes how the source code and executable files for the work
#  may be used by a user.
#- "Author" means the individual or entity that offers the work under
#  the terms of this license.
#- "Derivative work"/"Derivative project" means a work based upon the project
#  or upon the work and other pre-existing projects.
#- "Executable files" refer to the executables, binary files, configuration
#  and any required data files included in the project.
#- "Publisher" means the provider of the website, magazine, CD-ROM, DVD,
#  or other medium from or by which the project is obtained by you.
#- "Source code" refers to the collection of source code
#  and configuration files used to create the executable files.
#- "Standard version" refers to such a work if it has not been modified,
#  or has been modified in accordance with the consent of the author,
#  such consent being in the full discretion of the author.
#- "Work"/"Project" refers to the collection of files distributed by the
#  publisher, including the source code, executable files, binaries,
#  data files, documentation, whitepapers and the articles.
#- "You" is you, an individual or entity wishing to use the project
#  and exercise your rights under this license.
#
#
#
#II. Representations, Warranties, and Disclaimer:
#
#THIS WORK IS PROVIDED "AS IS", "WHERE IS" AND "AS AVAILABLE",
#WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OR CONDITIONS OR GUARANTEES.
#YOU, THE USER, ASSUME ALL RISK IN ITS USE, INCLUDING COPYRIGHT INFRINGEMENT,
#PATENT INFRINGEMENT, SUITABILITY, ETC.
#
#AUTHOR EXPRESSLY DISCLAIMS ALL EXPRESS, IMPLIED OR STATUTORY WARRANTIES
#OR CONDITIONS, INCLUDING WITHOUT LIMITATION, WARRANTIES OR CONDITIONS
#OF MERCHANTABILITY, MERCHANTABLE QUALITY OR FITNESS FOR A PARTICULAR PURPOSE,
#OR ANY WARRANTY OF TITLE OR NON-INFRINGEMENT,
#OR THAT THE WORK (OR ANY PORTION THEREOF) IS CORRECT, USEFUL, BUG-FREE
#OR FREE OF VIRUSES.
#
#YOU MUST PASS THIS DISCLAIMER ON WHENEVER YOU DISTRIBUTE THE WORK
#OR DERIVATIVE WORKS.
#
#
#
#III. Indemnity:
#
#YOU AGREE TO DEFEND, INDEMNIFY AND HOLD HARMLESS THE AUTHOR
#AND THE PUBLISHER FROM AND AGAINST ANY CLAIMS, SUITS, LOSSES, DAMAGES,
#LIABILITIES, COSTS, AND EXPENSES
#(INCLUDING REASONABLE LEGAL OR ATTORNEYS' FEES)
#RESULTING FROM OR RELATING TO ANY USE OF THE WORK BY YOU.
#
#
#
#IV. Limitation on Liability:
#
#EXCEPT TO THE EXTENT REQUIRED BY APPLICABLE LAW, IN NO EVENT WILL
#THE AUTHOR OR THE PUBLISHER BE LIABLE TO YOU ON ANY LEGAL THEORY
#FOR ANY SPECIAL, INCIDENTAL, CONSEQUENTIAL, PUNITIVE OR EXEMPLARY DAMAGES
#ARISING OUT OF THIS LICENSE OR THE USE OF THE WORK OR OTHERWISE,
#EVEN IF THE AUTHOR OR THE PUBLISHER HAS BEEN ADVISED OF THE POSSIBILITY
#OF SUCH DAMAGES.
#
#
#V. Publisher:
#
#THE PARTIES HEREBY CONFIRM THAT THE PUBLISHER SHALL NOT,
#UNDER ANY CIRCUMSTANCES, BE RESPONSIBLE FOR AND SHALL NOT HAVE ANY LIABILITY
#IN RESPECT OF THE SUBJECT MATTER OF THIS LICENSE.
#THE PUBLISHER MAKES NO WARRANTY WHATSOEVER IN CONNECTION WITH THE WORK
#AND SHALL NOT BE LIABLE TO YOU OR ANY PARTY ON ANY LEGAL THEORY
#FOR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION ANY GENERAL,
#SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING IN CONNECTION
#TO THIS LICENSE.
#
#THE PUBLISHER RESERVES THE RIGHT TO CEASE MAKING THE WORK AVAILABLE TO YOU
#AT ANY TIME WITHOUT NOTICE
#
#
#
#VI. Copying, Distribution, and Modification:
#
#YOU CAN COPY, DISTRIBUTE VERBATIM OR MODIFIED VERSIONS OF THIS SOFTWARE,
#AS LONG AS YOU ADHERE TO THE FOLLOWING TERMS:
#
#                     DARKGEEM PUBLIC LICENSE
#  TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION
#
#  1. You agree not to remove any of the original copyright, patent,
#     trademark, and attribution notices and associated disclaimers
#     that may appear in this project.
#  2. You agree not to advertise or in any way imply that this work
#     is a product of your own.
#  3. The name of the author may not be used to endorse or promote
#     products derived from this project without the prior written
#     consent of the author.
#  4. You agree not to sell, lease, or rent any part of this project.
#     This does not restrict you from including this project or
#     any part of this project inside a larger software distribution
#     that itself is being sold.
#     This project by itself, though, cannot be sold, leased or rented.
#  5. You may distribute the executable files and source code
#     only under the terms of this license, and you must include
#     a copy of, or the Uniform Resource Identifier for, this license
#     with every copy of the executable files or source code
#     you distribute and ensure that anyone receiving such
#     executable files and source code agrees that the terms
#     of this cicense apply to such executable files and/or source code.
#  6. You may not offer or impose any terms on this project
#     that alter or restrict the terms of this license
#     or the recipients' exercise of the rights granted hereunder.
#  7. You may not sublicense this project.
#  8. You must keep intact all notices that refer to this License
#     and to the disclaimer of warranties.
#  9. You agree not to use this project for illegal,
#     immoral or improper purposes, or on pages containing illegal,
#     immoral or improper material.
#     This project is subject to applicable export laws.
#     You agree to comply with all such laws and regulations
#     that may apply to this project after your receipt of this project.
""".replace('\n#', '\n# ').strip()
